// This file is part of Virtdbg
// Copyright (C) 2010-2011 Damien AUMAITRE

//  Licence is GPLv3, see LICENCE.txt in the top-level directory


#include "driver.h"

NTSTATUS DriverUnload(PDRIVER_OBJECT DriverObject)
{
    CCHAR i;
    KIRQL OldIrql;
    KAFFINITY OldAffinity;

	DbgPrint("DriverUnload\n");
    
	// for each processor stop the virtualization before module unload
    for (i=0; i<KeNumberProcessors; i++)
    {
        OldAffinity = KeSetSystemAffinityThreadEx((KAFFINITY)(1<<i));

		/* KeraiseIrqlToTpcLevel:
		 *	raises the hardware priority to IRQL = DISPATCH_LEVEL, thereby masking off
		 *	interrupts of equivalent or lower IRQL on the current processor.
		 *	Returns the previous IRQL for re-setting later.
		*/
        OldIrql = KeRaiseIrqlToDpcLevel();

        _StopVirtualization();
        KeLowerIrql(OldIrql);
        KeRevertToUserAffinityThreadEx(OldAffinity);
    }

    return STATUS_SUCCESS;
}

NTSTATUS DriverEntry(PDRIVER_OBJECT DriverObject, PUNICODE_STRING RegistryPath)
{
    NTSTATUS Status;
    HANDLE hThread = NULL;
    
	DriverObject->DriverUnload = DriverUnload;
	DbgPrint("DriverLoad\n");
	
    Status = PsCreateSystemThread(&hThread, 
                                  THREAD_ALL_ACCESS, 
                                  NULL, 
                                  NULL, 
                                  NULL, 
                                  VirtDbgStart, NULL);
    if (!NT_SUCCESS(Status))
    {
        return Status;
    }
	

    return STATUS_SUCCESS;
}


