#include <ntddk.h>
#include "cpuid.h"

char SizeOfPhysicalAddressRange();

void DriverUnload(PDRIVER_OBJECT pDriverObject)
{
	DbgPrint("Driver unloading\n");
}

NTSTATUS DriverEntry(PDRIVER_OBJECT DriverObject, PUNICODE_STRING RegistryPath) 
{
	CpuId_t regs; // eax, ebx, ecx, edx
	int regEAX;
	char cpusig[16];
	unsigned __int64 tsc;

	DriverObject->DriverUnload = DriverUnload;
	RtlZeroMemory(cpusig, sizeof(cpusig));

	DbgPrint("KernelSpace CPUID Lab + rdtsc Loaded!\n");

	for(regEAX=0; regEAX < 6; ++regEAX) {
		__cpuid(regs.i, regEAX);
		DbgPrint("eax=%#010x: %#010x %#010x %#010x %#010x\n", 
			regEAX, regs.eax, regs.ebx, regs.ecx, regs.edx);

		if (0 == regEAX) {
			RtlCopyMemory(cpusig,	(void*) &(regs.ebx), 4);
			RtlCopyMemory(cpusig+4, (void*) &(regs.edx), 4);
			RtlCopyMemory(cpusig+8, (void*) &(regs.ecx), 4);
			DbgPrint("CpuSig: %s\n", cpusig);
		}
	}

	DbgPrint("SizeOfPhysicalAddressRange: %d\n", SizeOfPhysicalAddressRange());
	tsc = __rdtsc();
	DbgPrint("TSC: %I64d\n", tsc);
	DbgPrint("CPUID Lab done. Unload it...\n");

	return STATUS_SUCCESS;

}

char SizeOfPhysicalAddressRange()
{
	int eax;
	CpuId_t regs; // eax, ebx, ecx, edx
	char bitres;

	/* first check for 64-bit: CPUID.80000001:EDX[29] = 1
	* See 3.3.1 of Volume 3  (System Programming Guide)
	*/
	eax = 0x80000001;
	__cpuid(regs.i, eax);
	bitres = _bittest((long*) &regs.edx, 29);
	if (bitres > 0) {
		/* physical address range is implementation-specific and 
		* indicated by CPUID.80000008H:EAX[bits 7-0]
		* Section 3.3.1 of Volume 3 (System Programming Guide)
		*/
		eax = 0x80000008;
		__cpuid(regs.i, eax);

		return regs.eax & 0xff;
	} else {
		DbgPrint("not 64-bit!\n");
	}

	return 0;
}
